<?php

namespace EventPortal\Tests\Unit\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Actions\UpdateSteps;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionOption;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Form\ValueObjects\StepData;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Collection;

class UpdateStepsTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_update_form_step(): void
    {
        $form = factory(Form::class)->create();
        factory(FormStep::class)->create(['form_id' => $form->id]);

        (new UpdateSteps)->execute($this->getUpdatedStep(), $form);

        $step = $form->steps->first();
        $this->assertEquals('Updated test stage', $step->name);
        $this->assertEquals('Updated test uuid', $step->uuid);
        $this->assertEquals(1, $step->order);
    }

    private function getUpdatedStep(array $override = []): Collection
    {
        return collect(array_merge([
            [
                'id' => 1,
                'name' => 'Updated test stage',
                'order' => 1,
                'uuid' => 'Updated test uuid',
            ],
        ], $override))->mapInto(StepData::class);
    }

    /** @test **/
    public function can_add_new_form_step(): void
    {
        $form = factory(Form::class)->create();
        factory(FormStep::class)->create(['form_id' => $form->id]);

        (new UpdateSteps)->execute($this->getNewStep(), $form);

        $step = $form->steps->first();
        $this->assertEquals('New test stage', $step->name);
        $this->assertEquals('New test uuid', $step->uuid);
        $this->assertEquals(1, $step->order);
    }

    private function getNewStep(array $override = []): Collection
    {
        return collect(array_merge([
            [
                'id' => 1,
                'name' => 'New test stage',
                'order' => 1,
                'uuid' => 'New test uuid',
            ],
        ], $override))->mapInto(StepData::class);
    }

    /** @test **/
    public function deletes_unused_form_step(): void
    {
        $form = factory(Form::class)->create();
        factory(FormStep::class)->create(['form_id' => $form->id]);

        (new UpdateSteps)->execute(collect([]), $form);

        $this->assertCount(0, $form->steps);
    }

    /** @test **/
    public function deletes_sections_questions_options_validations_when_deleting_step(): void
    {
        $form = factory(Form::class)->create();
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)->create(['form_step_section_id' => $section->id]);
        factory(QuestionOption::class)->create(['question_id' => $question->id]);
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        (new UpdateSteps)->execute(collect([]), $form);

        $this->assertCount(0, FormStepSection::all());
        $this->assertCount(0, Question::all());
        $this->assertCount(0, QuestionOption::all());
        $this->assertCount(0, QuestionValidation::all());
    }
}
